import os
from django.contrib.auth import get_user_model
from uuid import uuid4
from datetime import datetime
from slugify import slugify
from firebase_admin import messaging
CustomUser = get_user_model()

def custom_upload_to(instance, filename, base_folder):
    """Generates a custom upload path based on the base folder and unique filename."""
    ext = os.path.splitext(filename)[-1].lower()
    unique_filename = f"{datetime.now().strftime('%Y%m%d%H%M%S')}_{uuid4().hex}{ext}"
    return f"{base_folder}/{unique_filename}"

def project_upload_to(instance, filename):
    """Upload project files (images, videos, others) to the user-specific folder under project/{project_title_slug}/{folder}."""
    project_title_slug = slugify(instance.project_title)
    extension = os.path.splitext(filename)[1].lower()
    
    if extension in ['.jpg', '.jpeg', '.png', '.gif']:
        folder = 'images'
    elif extension in ['.mp4', '.avi', '.mov']:
        folder = 'videos'
    else:
        folder = 'others'
    
    user_folder = f"profiles/{instance.user.id}_{instance.user.username}/project/{project_title_slug}/{folder}"
    return custom_upload_to(instance, filename, user_folder)

def pan_card_upload_to(instance, filename):
    """Upload pan card files to the user-specific folder under events/pan_cards."""
    return f"profiles/{instance.user.id}_{instance.user.username}/events/pan_cards/{filename}"

def cancelled_cheque_upload_to(instance, filename):
    """Upload cancelled cheque files to the user-specific folder under events/cheques."""
    return f"profiles/{instance.user.id}_{instance.user.username}/events/cheques/{filename}"

def event_banner_upload_to(instance, filename):
    """Upload event banner files to the user-specific folder under events/banners."""
    return f"profiles/{instance.user.id}_{instance.user.username}/events/banners/{filename}"

def internship_document_upload_to(instance, filename):
    """Upload internship documents to the user-specific folder under internships/documents."""
    return f"profiles/{instance.user.id}_{instance.user.username}/internships/documents/{filename}"

def apprenticeship_document_upload_to(instance, filename):
    """Upload apprenticeship documents to the user-specific folder under apprenticeships/documents."""
    return f"profiles/{instance.user.id}_{instance.user.username}/apprenticeships/documents/{filename}"

def event_document_upload_to(instance, filename):
    """Upload event documents to the user-specific folder under events/documents."""
    return f"profiles/{instance.user.id}_{instance.user.username}/events/documents/{filename}"

def gigwork_document_upload_to(instance, filename):
    """Upload gig work documents to the user-specific folder under gigwork/documents."""
    return f"profiles/{instance.user.id}_{instance.user.username}/gigwork/documents/{filename}"

def castingcall_document_upload_to(instance, filename):
    return f"profiles/{instance.user.id}_{instance.user.username}/castingcall/scripts/{filename}"

def castingcall_photos_upload_to(instance, filename):
    """Upload casting call photos to the user-specific folder under castingcall/photos."""
    return f"profiles/{instance.user.id}_{instance.user.username}/castingcall/photos/{filename}"

def press_release_upload_to(instance, filename):
    """Upload press release files to the user-specific folder under project/press_release."""
    return  f"profiles/{instance.user.id}_{instance.user.username}/project/press_release"

def media_release_upload_to(instance, filename):
    """Upload media release files to the user-specific folder under project/media_release."""
    return f"profiles/{instance.user.id}_{instance.user.username}/project/media_release"


def headshot_form_upload_to(instance,filename):
    return f"profiles/{instance.user.id}_{instance.user.username}/project/headshot"

def send_push_notification(self, user_id, message):
    user = CustomUser.objects.get(id=user_id)
    fcm_token = user.fcm_token  # Assuming you have a field for storing FCM tokens
    
    # Create the message
    notification = messaging.Message(
        notification=messaging.Notification(
            title="Casting Call Update",
            body=message
        ),
        token=fcm_token
    )
    
    # Send the notification
    messaging.send(notification)
    print(f"Notification sent to User {user_id}: {message}")

def cover_image_upload_to(instance, filename):
    # project_title_slug = slugify(instance.project_title)
    return f"profiles/{instance.user.id}_{instance.user.username}/project/{filename}/cover_image"

def thumbnails_upload_to(instance, filename):
    # project_title_slug = slugify(instance.project_title)
    return f"profiles/{instance.user.id}_{instance.user.username}/project/{filename}/thumbnails"